/*!
* Moneypot Bucket API JS Library
*
* @author Dexon Noxed (https://dexontech.net/)
* @license Open source under the MIT License.
* @homepage https://github.com/dexon95/mp-bucket#readme
* 
* Docs: https://portal.moneypot.com/docs/v2/
*
* https://www.moneypot.com/
*/

"use strict";

const https = require('https');
const url = require("url");

const mpApiEndpoint = 'api.moneypot.com';

const mpBucketApiLib = (() => {
  let self = this;

  self.listBuckets = function(obj, callback){
    try{
      assert(typeof obj == "object", '[listBuckets] No object provided');
      assert(typeof obj.params == "object", '[listBuckets] No parameters provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[listBuckets] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[listBuckets] Parameter \'app_secret\' (string) is required.');

      Get('/v2/buckets', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  self.bucketInfo = function(obj, callback){
    try{
      assert(typeof obj == "object", '[bucketInfo] No object provided');
      assert(typeof obj.params == "object", '[bucketInfo] No parameters provided');
      assert(typeof obj.fields == "object", '[bucketInfo] No fields provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[bucketInfo] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[bucketInfo] Parameter \'app_secret\' (string) is required.');
      assert(typeof obj.fields.bucket_uuid == "string" && obj.fields.bucket_uuid.length > 0, '[bucketInfo] Field \'bucket_uuid\' (string) is required.');

      Get('/v2/buckets/single', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  self.createBucket = function(obj, callback){
    try{
      assert(typeof obj == "object", '[createBucket] No object provided');
      assert(typeof obj.params == "object", '[createBucket] No parameters provided');
      assert(typeof obj.fields == "object", '[createBucket] No fields provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[createBucket] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[createBucket] Parameter \'app_secret\' (string) is required.');
      assert(typeof obj.fields.bucket_name == "string" && obj.fields.bucket_name.length > 0, '[createBucket] Field \'bucket_name\' (string) is required.');
      assert(typeof obj.fields.bucket_type == "string" && obj.fields.bucket_type.length > 0, '[createBucket] Field \'bucket_type\' (string) is required.');
      assert(obj.fields.bucket_type == "closed" || obj.fields.bucket_type == "open" || obj.fields.bucket_type == "strict", '[createBucket] \''+obj.fields.bucket_type+'\' is not a valid bucket type. Valid types: \'closed\' \'open\' \'strict\'');
      assert(typeof obj.fields.coin == "string" && obj.fields.coin.length > 0, '[createBucket] Field \'coin\' (string) is required.');

      Post('/v2/buckets/create', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  self.destroyBucket = function(obj, callback){
    try{
      assert(typeof obj == "object", '[destroyBucket] No object provided');
      assert(typeof obj.params == "object", '[destroyBucket] No parameters provided');
      assert(typeof obj.fields == "object", '[destroyBucket] No fields provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[destroyBucket] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[destroyBucket] Parameter \'app_secret\' (string) is required.');
      assert(typeof obj.fields.bucket_uuid == "string" && obj.fields.bucket_uuid.length > 0, '[destroyBucket] Field \'bucket_uuid\' (string) is required.');

      Post('/v2/buckets/destroy', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  self.creditBucket = function(obj, callback){
    try{
      assert(typeof obj == "object", '[creditBucket] No object provided');
      assert(typeof obj.params == "object", '[creditBucket] No parameters provided');
      assert(typeof obj.fields == "object", '[creditBucket] No fields provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[creditBucket] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[creditBucket] Parameter \'app_secret\' (string) is required.');
      assert(typeof obj.fields.bucket_uuid == "string" && obj.fields.bucket_uuid.length > 0, '[creditBucket] Field \'bucket_uuid\' (string) is required.');
      assert(typeof obj.fields.auth_id == "string" && obj.fields.auth_id.length > 0, '[creditBucket] Field \'auth_id\' (string) is required.');
      assert(typeof obj.fields.amount == "number" && obj.fields.amount > 0, '[creditBucket] Field \'amount\' (number) is required.');

      Post('/v2/buckets/credit', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  self.debitBucket = function(obj, callback){
    try{
      assert(typeof obj == "object", '[debitBucket] No object provided');
      assert(typeof obj.params == "object", '[debitBucket] No parameters provided');
      assert(typeof obj.fields == "object", '[debitBucket] No fields provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[debitBucket] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[debitBucket] Parameter \'app_secret\' (string) is required.');
      assert(typeof obj.fields.bucket_uuid == "string" && obj.fields.bucket_uuid.length > 0, '[debitBucket] Field \'bucket_uuid\' (string) is required.');
      assert(typeof obj.fields.auth_id == "string" && obj.fields.auth_id.length > 0, '[debitBucket] Field \'auth_id\' (string) is required.');
      assert(typeof obj.fields.amount == "number" && obj.fields.amount > 0, '[debitBucket] Field \'amount\' (number) is required.');

      Post('/v2/buckets/debit', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  self.txInfo = function(obj, callback){
    try{
      assert(typeof obj == "object", '[txInfo] No object provided');
      assert(typeof obj.params == "object", '[txInfo] No parameters provided');
      assert(typeof obj.fields == "object", '[txInfo] No fields provided');
      assert(typeof obj.params.auth_id == "string" && obj.params.auth_id.length > 0, '[txInfo] Parameter \'auth_id\' (string) is required.');
      assert(typeof obj.params.app_secret == "string" && obj.params.app_secret.length > 0, '[txInfo] Parameter \'app_secret\' (string) is required.');
      assert(typeof obj.fields.bucket_tx_uuid  == "string" && obj.fields.bucket_tx_uuid .length > 0, '[txInfo] Field \'bucket_tx_uuid \' (string) is required.');

      Get('/v2/buckets/transaction', obj, (err, response) => {
        if(typeof callback == "function") callback(err, response);
      });
    }catch(e){
      if(typeof callback == "function") callback(e);
    }
  };

  return self;
})();

module.exports = mpBucketApiLib;

/**
 * https GET request function
 */
function Get(path, params, callback, host = null) {
  let linkParams = '?';
  for(let i in params.params) {
    linkParams += `${i}=${encodeURIComponent(params.params[i])}&`;
  }

  https.get("https://" + (!host?mpApiEndpoint:host) + path + linkParams.slice(0, -1), response => {
    if (response.statusCode > 300 && response.statusCode < 400 && response.headers.location) {
      if (url.parse(response.headers.location).pathname) {
        Get(url.parse(response.headers.location).pathname, params, callback, url.parse(response.headers.location).hostname);
      } else {
        if(typeof callback == "function") callback("Could not retrieve the pathname from the request url.");
      }
    } else {
      let body = '';
      response.on('data', d => {
        body += d;
      });
      response.on('end', () => {
        let err = null;

        try{
          body = JSON.parse(body);
        }catch(e){
          if(typeof callback == "function"){
            return callback(`Could not parse the response: ${body}`);
          }
        }

        if (response.statusCode < 200 || response.statusCode > 299) {
          if(typeof callback == "function"){
            return callback(`Error code ${response.statusCode} was returned. Body: ` + (typeof body !== "string" ? JSON.stringify(body) : body));
          }
        }

        if(typeof callback == "function") callback(err, body);
      });
    }
  }).on('error', e => {
    if(typeof callback == "function") callback(e);
  });
}

/**
 * https POST request function
 */
function Post(path, params, callback, host = null) {
  let linkParams = '?';
  for(let i in params.params) {
    linkParams += `${i}=${encodeURIComponent(params.params[i])}&`;
  }

  let req = https.request({
    host: (!host?mpApiEndpoint:host),
    path: path + linkParams.slice(0, -1),
    method: 'POST',
    header: {
      'Content-Type': 'application/json',
      'Content-Length': JSON.stringify(params.fields).length
    }
  }, response => {
    if (response.statusCode > 300 && response.statusCode < 400 && response.headers.location) {
      if (url.parse(response.headers.location).pathname) {
        Post(url.parse(response.headers.location).pathname, params, callback, url.parse(response.headers.location).hostname);
      } else {
        if(typeof callback == "function") callback("Could not retrieve the pathname from the request url.");
      }
    } else {
      let body = '';
      response.on('data', d => {
        body += d;
      });
      response.on('end', () => {
        let err = null;

        try{
          body = JSON.parse(body);
        }catch(e){
          if(typeof callback == "function"){
            return callback(`Could not parse the response: ${body}`);
          }
        }

        if (response.statusCode < 200 || response.statusCode > 299) {
          if(typeof callback == "function"){
            return callback(`Error code ${response.statusCode} was returned. Body: ` + (typeof body !== "string" ? JSON.stringify(body) : body));
          }
        }

        if(typeof callback == "function") callback(err, body);
      });
    }
  });

  req.write(JSON.stringify(params.fields));
  req.on('error', e => {
    if(typeof callback == "function") callback(e);
  });
  req.end();
}

/**
 * Assertion function
 */
function assert(condition, message) {
  if (!condition) {
    message = message || "Assertion failed";
    if (typeof Error !== "undefined") {
      throw new Error(message);
    }
    throw message; // Fallback
  }
}